@@ -379,8 +379,14 @@ class Checkout(EditModel):
 class BlamePaths(ContextCommand):
     """Blame view for paths."""
 
-    def __init__(self, context, paths):
+    @staticmethod
+    def name():
+        return N_('Blame...')
+
+    def __init__(self, context, paths=None):
         super(BlamePaths, self).__init__(context)
+        if not paths:
+            paths = context.selection.union()
         viewer = utils.shell_split(prefs.blame_viewer(context))
         self.argv = viewer + list(paths)
 
@@ -586,6 +592,12 @@ class Commit(ResetMode):
         return msg
 
 
+class CycleReferenceSort(ContextCommand):
+    """Choose the next reference sort type"""
+    def do(self):
+        self.model.cycle_ref_sort()
+
+
 class Ignore(ContextCommand):
     """Add files to .gitignore"""
 
@@ -829,16 +841,30 @@ class MoveToTrash(RemoveFiles):
         super(MoveToTrash, self).__init__(context, send2trash, filenames)
 
 
-class DeleteBranch(ContextCommand):
+class DeleteBranch(ConfirmAction):
     """Delete a git branch."""
 
     def __init__(self, context, branch):
         super(DeleteBranch, self).__init__(context)
         self.branch = branch
 
-    def do(self):
-        status, out, err = self.model.delete_branch(self.branch)
-        Interaction.log_status(status, out, err)
+    def confirm(self):
+        title = N_('Delete Branch')
+        question = N_('Delete branch "%s"?') % self.branch
+        info = N_('The branch will be no longer available.')
+        ok_txt = N_('Delete Branch')
+        return Interaction.confirm(title, question, info, ok_txt,
+                                   default=True, icon=icons.discard())
+
+    def action(self):
+        return self.model.delete_branch(self.branch)
+
+    def error_message(self):
+        return N_('Error deleting branch "%s"' % self.branch)
+
+    def command(self):
+        command = 'git branch -D %s'
+        return command % self.branch
 
 
 class Rename(ContextCommand):
@@ -891,25 +917,29 @@ class RenameBranch(ContextCommand):
         Interaction.log_status(status, out, err)
 
 
-class DeleteRemoteBranch(ContextCommand):
+class DeleteRemoteBranch(DeleteBranch):
     """Delete a remote git branch."""
 
     def __init__(self, context, remote, branch):
-        super(DeleteRemoteBranch, self).__init__(context)
+        super(DeleteRemoteBranch, self).__init__(context, branch)
         self.remote = remote
-        self.branch = branch
 
-    def do(self):
-        status, out, err = self.git.push(self.remote, self.branch, delete=True)
+    def action(self):
+        return self.git.push(self.remote, self.branch, delete=True)
+
+    def success(self):
         self.model.update_status()
+        Interaction.information(
+            N_('Remote Branch Deleted'),
+            N_('"%(branch)s" has been deleted from "%(remote)s".')
+            % dict(branch=self.branch, remote=self.remote))
 
-        title = N_('Error Deleting Remote Branch')
-        Interaction.command(title, 'git push', status, out, err)
-        if status == 0:
-            Interaction.information(
-                N_('Remote Branch Deleted'),
-                N_('"%(branch)s" has been deleted from "%(remote)s".')
-                % dict(branch=self.branch, remote=self.remote))
+    def error_message(self):
+        return N_('Error Deleting Remote Branch')
+
+    def command(self):
+        command = 'git push --delete %s %s'
+        return command % (self.remote, self.branch)
 
 
 def get_mode(model, staged, modified, unmerged, untracked):
@@ -1248,13 +1278,21 @@ class LaunchDifftool(ContextCommand):
                 cfg = self.cfg
                 cmd = cfg.terminal()
                 argv = utils.shell_split(cmd)
+
+                terminal = os.path.basename(argv[0])
+                shellquote_terms = set(['xfce4-terminal'])
+                shellquote_default = terminal in shellquote_terms
+
                 mergetool = ['git', 'mergetool', '--no-prompt', '--']
                 mergetool.extend(paths)
-                needs_shellquote = set(['gnome-terminal', 'xfce4-terminal'])
-                if os.path.basename(argv[0]) in needs_shellquote:
+                needs_shellquote = cfg.get(
+                    'cola.terminalshellquote', shellquote_default)
+
+                if needs_shellquote:
                     argv.append(core.list2cmdline(mergetool))
                 else:
                     argv.extend(mergetool)
+
                 core.fork(argv)
         else:
             difftool_run(self.context)
@@ -1829,10 +1867,9 @@ class RevertUnstagedEdits(RevertEditsCommand):
         return N_('Revert Unstaged Edits...')
 
     def checkout_from_head(self):
-        # If we are amending and a modified file is selected
-        # then we should include "HEAD^" on the command-line.
-        selected = self.selection.selection()
-        return not selected.staged and self.model.amending()
+        # Being in amend mode should not affect the behavior of this command.
+        # The only sensible thing to do is to checkout from the index.
+        return False
 
     def confirm(self):
         title = N_('Revert Unstaged Changes?')
