@@ -158,7 +158,7 @@ class Difftool(standard.Dialog):
         self.diff_all_button.setEnabled(has_selection)
 
     def tree_double_clicked(self, item, _column):
-        path = self.tree.filename_from_item(item)
+        path = filetree.filename_from_item(item)
         left, right = self._left_right_args()
         cmds.difftool_launch(
             self.context, left=left, right=right, paths=[path])
