@@ -1,44 +1,48 @@
 from __future__ import division, absolute_import, unicode_literals
-
 import time
-
-from PyQt4 import QtGui
-from PyQt4 import QtCore
-from PyQt4 import QtNetwork
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
-
-from cola import qtutils
-from cola import core
-from cola.compat import ustr, urllib
 import hashlib
 
+from qtpy import QtCore
+from qtpy import QtGui
+from qtpy import QtWidgets
+from qtpy import QtNetwork
+
+from . import core
+from . import icons
+from .compat import bstr
+from .compat import ustr
+from .compat import parse
+from .widgets import defs
+
 
 class Gravatar(object):
+
     @staticmethod
     def url_for_email(email, imgsize):
-        email_hash = hashlib.md5(core.encode(email)).hexdigest()
-        default_url = b'https://git-cola.github.io/images/git-64x64.jpg'
-        encoded_url = urllib.quote(default_url, b'')
-        query = '?s=%d&d=%s' % (imgsize, encoded_url)
+        email_hash = core.decode(hashlib.md5(core.encode(email)).hexdigest())
+        # Python2.6 requires byte strings for urllib2.quote() so we have
+        # to force
+        default_url = 'https://git-cola.github.io/images/git-64x64.jpg'
+        encoded_url = parse.quote(core.encode(default_url), core.encode(''))
+        query = '?s=%d&d=%s' % (imgsize, core.decode(encoded_url))
         url = 'https://gravatar.com/avatar/' + email_hash + query
         return url
 
 
-class GravatarLabel(QtGui.QLabel):
+class GravatarLabel(QtWidgets.QLabel):
+
     def __init__(self, parent=None):
-        QtGui.QLabel.__init__(self, parent)
+        QtWidgets.QLabel.__init__(self, parent)
 
         self.email = None
         self.response = None
         self.timeout = 0
-        self.imgsize = 48
+        self.imgsize = defs.medium_icon
         self.pixmaps = {}
+        self._default_pixmap_bytes = None
 
         self.network = QtNetwork.QNetworkAccessManager()
-        self.connect(self.network,
-                     SIGNAL('finished(QNetworkReply*)'),
-                     self.network_finished)
+        self.network.finished.connect(self.network_finished)
 
     def set_email(self, email):
         if email in self.pixmaps:
@@ -59,31 +63,33 @@ class GravatarLabel(QtGui.QLabel):
         self.network.get(QtNetwork.QNetworkRequest(QtCore.QUrl(url)))
 
     def default_pixmap_as_bytes(self):
-        xres = self.imgsize
-        pixmap = qtutils.git_icon().pixmap(xres)
-        byte_array = QtCore.QByteArray()
-        buf = QtCore.QBuffer(byte_array)
-        buf.open(QtCore.QIODevice.WriteOnly)
-        pixmap.save(buf, 'PNG')
-        buf.close()
+        if self._default_pixmap_bytes is None:
+            xres = self.imgsize
+            pixmap = icons.cola().pixmap(xres)
+            byte_array = QtCore.QByteArray()
+            buf = QtCore.QBuffer(byte_array)
+            buf.open(QtCore.QIODevice.WriteOnly)
+            pixmap.save(buf, 'PNG')
+            buf.close()
+            self._default_pixmap_bytes = byte_array
+        else:
+            byte_array = self._default_pixmap_bytes
         return byte_array
 
     def network_finished(self, reply):
         email = self.email
 
-        header = QtCore.QByteArray('Location')
-        raw_header = reply.rawHeader(header)
-        if raw_header:
-            location = ustr(QtCore.QString(raw_header)).strip()
-            request_location = ustr(
-                    Gravatar.url_for_email(self.email, self.imgsize))
+        header = QtCore.QByteArray(bstr('Location'))
+        location = ustr(reply.rawHeader(header)).strip()
+        if location:
+            request_location = Gravatar.url_for_email(self.email, self.imgsize)
             relocated = location != request_location
         else:
             relocated = False
 
         if reply.error() == QtNetwork.QNetworkReply.NoError:
             if relocated:
-                # We could do get_url(urllib.unquote(location)) to
+                # We could do get_url(parse.unquote(location)) to
                 # download the default image.
                 # Save bandwidth by using a pixmap.
                 self.response = self.default_pixmap_as_bytes()
@@ -101,10 +107,12 @@ class GravatarLabel(QtGui.QLabel):
         # email address.  We can't blindly trust self.email else
         # we may add cache entries for thee wrong email address.
         url = Gravatar.url_for_email(email, self.imgsize)
-        if url == ustr(reply.url().toString()):
+        if url == reply.url().toString():
             self.pixmaps[email] = pixmap
 
     def set_pixmap_from_response(self):
+        if self.response is None:
+            self.response = self._default_pixmap_bytes()
         pixmap = QtGui.QPixmap()
         pixmap.loadFromData(self.response)
         self.setPixmap(pixmap)
