@@ -7,6 +7,7 @@ import os
 
 from .. import core
 from .. import gitcmds
+from .. import version
 from ..git import STDOUT
 from ..observable import Observable
 from . import prefs
@@ -32,6 +33,8 @@ class MainModel(Observable):
     message_images_changed = 'images_changed'
     message_mode_about_to_change = 'mode_about_to_change'
     message_mode_changed = 'mode_changed'
+    message_submodules_changed = 'message_submodules_changed'
+    message_refs_updated = 'message_refs_updated'
     message_updated = 'updated'
 
     # States
@@ -93,7 +96,9 @@ class MainModel(Observable):
         self.staged_deleted = set()
         self.unstaged_deleted = set()
         self.submodules = set()
+        self.submodules_list = []
 
+        self.ref_sort = 0  # (0: version, 1:reverse-chrono)
         self.local_branches = []
         self.remote_branches = []
         self.tags = []
@@ -222,9 +227,9 @@ class MainModel(Observable):
         self._update_files(update_index=update_index)
         self._update_remotes()
         self._update_branches_and_tags()
-        self._update_branch_heads()
         self._update_commitmsg()
         self.update_config()
+        self.update_submodules_list()
         self.emit_updated()
 
     def update_config(self, emit=False, reset=False):
@@ -273,17 +278,21 @@ class MainModel(Observable):
     def _update_remotes(self):
         self.remotes = self.git.remote()[STDOUT].splitlines()
 
-    def _update_branch_heads(self):
-        # Set these early since they are used to calculate 'upstream_changed'.
-        self.currentbranch = gitcmds.current_branch(self.context)
-
     def _update_branches_and_tags(self):
         context = self.context
+        sort_types = (
+            'version:refname',
+            '-committerdate',
+        )
+        sort_key = sort_types[self.ref_sort]
         local_branches, remote_branches, tags = gitcmds.all_refs(
-            context, split=True)
+            context, split=True, sort_key=sort_key)
         self.local_branches = local_branches
         self.remote_branches = remote_branches
         self.tags = tags
+        # Set these early since they are used to calculate 'upstream_changed'.
+        self.currentbranch = gitcmds.current_branch(self.context)
+        self.notify_observers(self.message_refs_updated)
 
     def _update_merge_rebase_status(self):
         merge_head = self.git.git_path('MERGE_HEAD')
@@ -315,6 +324,10 @@ class MainModel(Observable):
             self._auto_commitmsg = ''
             self.set_commitmsg(self._prev_commitmsg)
 
+    def update_submodules_list(self):
+        self.submodules_list = gitcmds.list_submodule(self.context)
+        self.notify_observers(self.message_submodules_changed)
+
     def update_remotes(self):
         self._update_remotes()
         self._update_branches_and_tags()
@@ -328,7 +341,6 @@ class MainModel(Observable):
         status, out, err = self.git.branch(branch, new_branch, M=True)
         self.emit_about_to_update()
         self._update_branches_and_tags()
-        self._update_branch_heads()
         self.emit_updated()
         return status, out, err
 
@@ -337,16 +349,18 @@ class MainModel(Observable):
         return gitcmds.remote_url(self.context, name, push=push)
 
     def fetch(self, remote, **opts):
-        return run_remote_action(self.git.fetch, remote, **opts)
+        return run_remote_action(self.context, self.git.fetch, remote, **opts)
 
     def push(self, remote, remote_branch='', local_branch='', **opts):
         # Swap the branches in push mode (reverse of fetch)
         opts.update(dict(local_branch=remote_branch,
                          remote_branch=local_branch))
-        return run_remote_action(self.git.push, remote, push=True, **opts)
+        return run_remote_action(self.context, self.git.push, remote,
+                                 push=True, **opts)
 
     def pull(self, remote, **opts):
-        return run_remote_action(self.git.pull, remote, pull=True, **opts)
+        return run_remote_action(self.context, self.git.pull, remote,
+                                 pull=True, **opts)
 
     def create_branch(self, name, base, track=False, force=False):
         """Create a branch named 'name' from revision 'base'
@@ -370,12 +384,6 @@ class MainModel(Observable):
             errs.append(err)
         return (status, '\n'.join(outs), '\n'.join(errs))
 
-    def pad(self, pstr, num=22):
-        topad = num-len(pstr)
-        if topad > 0:
-            return pstr + ' '*topad
-        return pstr
-
     def is_commit_published(self):
         """Return True if the latest commit exists in any remote branch"""
         return bool(self.git.branch(r=True, contains='HEAD')[STDOUT])
@@ -392,9 +400,21 @@ class MainModel(Observable):
             return self.directory
         return core.getcwd()
 
+    def cycle_ref_sort(self):
+        """Choose the next ref sort type (version, reverse-chronological)"""
+        self.set_ref_sort(self.ref_sort + 1)
+
+    def set_ref_sort(self, raw_value):
+        value = raw_value % 2  # Currently two sort types
+        if value == self.ref_sort:
+            return
+        self.ref_sort = value
+        self._update_branches_and_tags()
+
 
 # Helpers
-def remote_args(remote,
+# pylint: disable=too-many-arguments
+def remote_args(context, remote,
                 local_branch='',
                 remote_branch='',
                 ff_only=False,
@@ -424,7 +444,11 @@ def remote_args(remote,
         elif no_ff:
             kwargs['no_ff'] = True
     elif force:
-        kwargs['force'] = True
+        # pylint: disable=simplifiable-if-statement
+        if push and version.check_git(context, 'force-with-lease'):
+            kwargs['force_with_lease'] = True
+        else:
+            kwargs['force'] = True
 
     if push and set_upstream:
         kwargs['set_upstream'] = True
@@ -453,6 +477,6 @@ def refspec_arg(local_branch, remote_branch, pull, push):
     return what
 
 
-def run_remote_action(action, remote, **kwargs):
-    args, kwargs = remote_args(remote, **kwargs)
+def run_remote_action(context, action, remote, **kwargs):
+    args, kwargs = remote_args(context, remote, **kwargs)
     return action(*args, **kwargs)
