@@ -187,6 +187,10 @@ class RepoTreeView(standard.TreeView):
             N_('Edit selected paths'),
             cmds.run(cmds.LaunchEditor, context), hotkeys.EDIT)
 
+        self.action_blame = qtutils.add_action_with_status_tip(
+            self, cmds.BlamePaths.name(), N_('Blame selected paths'),
+            cmds.run(cmds.BlamePaths, context))
+
         self.action_refresh = common.refresh_action(context, self)
 
         if not utils.is_win32():
@@ -196,8 +200,8 @@ class RepoTreeView(standard.TreeView):
             self.action_parent_dir = common.parent_dir_action(
                 context, self, self.selected_paths)
 
-            self.action_terminal = common.terminal_action(
-                context, self, self.selected_paths)
+        self.action_terminal = common.terminal_action(
+            context, self, self.selected_paths)
 
         self.x_width = QtGui.QFontMetrics(self.font()).width('x')
         self.size_columns()
@@ -347,6 +351,8 @@ class RepoTreeView(standard.TreeView):
         if not utils.is_win32():
             self.action_default_app.setEnabled(selected)
             self.action_parent_dir.setEnabled(selected)
+
+        if self.action_terminal is not None:
             self.action_terminal.setEnabled(selected)
 
         self.action_stage.setEnabled(staged or unstaged)
@@ -367,6 +373,7 @@ class RepoTreeView(standard.TreeView):
         menu.addAction(self.action_history)
         menu.addAction(self.action_difftool)
         menu.addAction(self.action_difftool_predecessor)
+        menu.addAction(self.action_blame)
         menu.addSeparator()
         menu.addAction(self.action_revert_unstaged)
         menu.addAction(self.action_revert_uncommitted)
@@ -376,6 +383,8 @@ class RepoTreeView(standard.TreeView):
             menu.addSeparator()
             menu.addAction(self.action_default_app)
             menu.addAction(self.action_parent_dir)
+
+        if self.action_terminal is not None:
             menu.addAction(self.action_terminal)
         menu.exec_(self.mapToGlobal(event.pos()))
 
@@ -709,13 +718,13 @@ class GitFileTreeModel(QtGui.QStandardItemModel):
         except KeyError:
             parent = dir_entries[dirname] = self.create_dir_entry(dirname)
 
-        row_items = self.create_row(path, False)
+        row_items = create_row(path, False)
         parent.appendRow(row_items)
 
     def add_directory(self, parent, path):
         """Add a directory entry to the model."""
         # Create model items
-        row_items = self.create_row(path, True)
+        row_items = create_row(path, True)
 
         try:
             parent_path = parent.path
@@ -734,10 +743,6 @@ class GitFileTreeModel(QtGui.QStandardItemModel):
 
         return row_items[0]
 
-    def create_row(self, path, is_dir):
-        """Return a list of items representing a row."""
-        return [GitTreeItem(path, is_dir)]
-
     def create_dir_entry(self, dirname):
         """
         Create a directory entry for the model.
@@ -763,6 +768,11 @@ class GitFileTreeModel(QtGui.QStandardItemModel):
         return parent
 
 
+def create_row(path, is_dir):
+    """Return a list of items representing a row."""
+    return [GitTreeItem(path, is_dir)]
+
+
 class GitTreeModel(GitFileTreeModel):
 
     def __init__(self, context, ref, parent):
