@@ -7,6 +7,7 @@ from qtpy import QtWidgets
 from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
 
+from ..models import prefs
 from .. import core
 from .. import gitcmds
 from .. import icons
@@ -65,6 +66,10 @@ class BranchValidator(QtGui.QValidator):
         return (state, string, idx)
 
 
+def _is_case_sensitive(text):
+    return bool([char for char in text if char.isupper()])
+
+
 class CompletionLineEdit(HintedLineEdit):
     """A lineedit with advanced completion abilities"""
 
@@ -134,16 +139,13 @@ class CompletionLineEdit(HintedLineEdit):
         """Return the completer's popup"""
         return self._completer.popup()
 
-    def _is_case_sensitive(self, text):
-        return bool([char for char in text if char.isupper()])
-
     def _text_changed(self, full_text):
         match_text = self._last_word()
         self._do_text_changed(full_text, match_text)
         self.complete_last_word()
 
     def _do_text_changed(self, full_text, match_text):
-        case_sensitive = self._is_case_sensitive(match_text)
+        case_sensitive = _is_case_sensitive(match_text)
         if case_sensitive:
             self._completer.setCaseSensitivity(Qt.CaseSensitive)
         else:
@@ -153,7 +155,7 @@ class CompletionLineEdit(HintedLineEdit):
 
     def update_matches(self):
         text = self._last_word()
-        case_sensitive = self._is_case_sensitive(text)
+        case_sensitive = _is_case_sensitive(text)
         self._completer.setCompletionPrefix(text)
         self._completer.model().update_matches(case_sensitive)
 
@@ -415,7 +417,7 @@ class CompletionModel(QtGui.QStandardItemModel):
         if not self.update_thread.isRunning():
             self.update_thread.start()
 
-    # pylint: disable=unused-argument
+    # pylint: disable=unused-argument,no-self-use
     def gather_matches(self, case_sensitive):
         return ((), (), set())
 
@@ -532,6 +534,7 @@ class GitCompletionModel(CompletionModel):
         except RuntimeError:  # C++ object has been deleted
             self.dispose()
 
+    # pylint: disable=no-self-use
     def matches(self):
         return []
 
@@ -622,6 +625,7 @@ class GitPathCompletionModel(GitCompletionModel):
     def __init__(self, context, parent):
         GitCompletionModel.__init__(self, context, parent)
 
+    # pylint: disable=no-self-use
     def candidate_paths(self):
         return []
 
@@ -673,8 +677,12 @@ class GitLogCompletionModel(GitRefCompletionModel):
         GitRefCompletionModel.__init__(self, context, parent)
         self.model_updated.connect(self.gather_paths, type=Qt.QueuedConnection)
         self._paths = []
+        self._model = context.model
 
     def gather_paths(self):
+        if not self._model.cfg.get(prefs.AUTOCOMPLETE_PATHS, True):
+            self._paths = []
+            return
         context = self.context
         self._paths = gitcmds.tracked_files(context)
 
@@ -754,7 +762,7 @@ class GitDialog(QtWidgets.QDialog):
         qtutils.connect_button(self.close_button, self.reject)
 
         self.setFocusProxy(self.lineedit)
-        self.lineedit.setFocus(True)
+        self.lineedit.setFocus()
 
     def text(self):
         return self.lineedit.text()
@@ -781,7 +789,7 @@ class GitDialog(QtWidgets.QDialog):
             dlg.lineedit.popup().move(mapped.x(), mapped.y())
             dlg.lineedit.popup().show()
             dlg.lineedit.refresh()
-            dlg.lineedit.setFocus(True)
+            dlg.lineedit.setFocus()
 
         QtCore.QTimer().singleShot(100, show_popup)
 
