@@ -33,6 +33,7 @@ from . import action
 from . import archive
 from . import bookmarks
 from . import branch
+from . import submodules
 from . import browse
 from . import cfgactions
 from . import clone
@@ -119,6 +120,13 @@ class MainView(standard.MainWindow):
         self.branchwidget = self.branchdock.widget()
         titlebar = self.branchdock.titleBarWidget()
         titlebar.add_corner_widget(self.branchwidget.filter_button)
+        titlebar.add_corner_widget(self.branchwidget.sort_order_button)
+
+        # "Submodule" widgets
+        self.submodulesdock = create_dock(
+            N_('Submodules'), self,
+            fn=partial(submodules.SubmodulesWidget, context))
+        self.submoduleswidget = self.submodulesdock.widget()
 
         # "Commit Message Editor" widget
         self.position_label = QtWidgets.QLabel()
@@ -174,7 +182,7 @@ class MainView(standard.MainWindow):
         self.unstage_selected_action.setIcon(icons.remove())
 
         self.show_diffstat_action = add_action(
-            self, N_('Diffstat'), cmds.run(cmds.Diffstat, context),
+            self, N_('Diffstat'), self.statuswidget.select_header,
             hotkeys.DIFFSTAT)
 
         self.stage_modified_action = add_action(
@@ -243,7 +251,7 @@ class MainView(standard.MainWindow):
 
         self.save_tarball_action = add_action(
             self, N_('Save As Tarball/Zip...'),
-            partial(archive.save_archive, context, self))
+            partial(archive.save_archive, context))
 
         self.quit_action = add_action(
             self, N_('Quit'), self.close, hotkeys.QUIT)
@@ -258,6 +266,10 @@ class MainView(standard.MainWindow):
         self.merge_abort_action = add_action(
             self, N_('Abort Merge...'), cmds.run(cmds.AbortMerge, context))
 
+        self.update_submodules_action = add_action(
+            self, N_('Update All Submodules...'),
+            cmds.run(cmds.SubmodulesUpdate, context))
+
         self.fetch_action = add_action(
             self, N_('Fetch...'), partial(remote.fetch, context),
             hotkeys.FETCH)
@@ -485,6 +497,8 @@ class MainView(standard.MainWindow):
         self.actions_menu.addAction(self.merge_local_action)
         self.actions_menu.addAction(self.merge_abort_action)
         self.actions_menu.addSeparator()
+        self.actions_menu.addAction(self.update_submodules_action)
+        self.actions_menu.addSeparator()
         self.actions_reset_menu = self.actions_menu.addMenu(N_('Reset'))
         self.actions_reset_menu.addAction(self.reset_branch_head_action)
         self.actions_reset_menu.addAction(self.reset_worktree_action)
@@ -546,6 +560,10 @@ class MainView(standard.MainWindow):
         self.view_menu.aboutToShow.connect(
             lambda: self.build_view_menu(self.view_menu))
         self.setup_dockwidget_view_menu()
+        if utils.is_darwin():
+            # TODO or self.menubar.setNativeMenuBar(False)
+            # Since native OSX menu doesn't show empty entries
+            self.build_view_menu(self.view_menu)
 
         # Help Menu
         self.help_menu = add_menu(N_('Help'), self.menubar)
@@ -562,9 +580,17 @@ class MainView(standard.MainWindow):
         if self.browser_dockable:
             self.addDockWidget(top, self.browserdock)
             self.tabifyDockWidget(self.browserdock, self.commitdock)
-        self.addDockWidget(top, self.bookmarksdock)
+
         self.addDockWidget(top, self.branchdock)
+        self.addDockWidget(top, self.submodulesdock)
+        self.addDockWidget(top, self.bookmarksdock)
         self.addDockWidget(top, self.recentdock)
+
+        self.tabifyDockWidget(self.branchdock, self.submodulesdock)
+        self.tabifyDockWidget(self.submodulesdock, self.bookmarksdock)
+        self.tabifyDockWidget(self.bookmarksdock, self.recentdock)
+        self.branchdock.raise_()
+
         self.addDockWidget(bottom, self.diffdock)
         self.addDockWidget(bottom, self.actionsdock)
         self.addDockWidget(bottom, self.logdock)
@@ -674,7 +700,8 @@ class MainView(standard.MainWindow):
             self.actionsdock,
             self.bookmarksdock,
             self.recentdock,
-            self.branchdock
+            self.branchdock,
+            self.submodulesdock
         ]
         if self.browser_dockable:
             dockwidgets.append(self.browserdock)
@@ -720,7 +747,11 @@ class MainView(standard.MainWindow):
             self.commiteditor.setFont(font)
 
         elif config == prefs.TABWIDTH:
-            # variable-tab-width setting
+            # This can be set locally or globally, so we have to use the
+            # effective value otherwise we'll update when we shouldn't.
+            # For example, if this value is overridden locally, and the
+            # global value is tweaked, we should not update.
+            value = prefs.tabwidth(self.context)
             self.diffeditor.set_tabwidth(value)
             self.commiteditor.set_tabwidth(value)
 
@@ -735,7 +766,8 @@ class MainView(standard.MainWindow):
             self.bookmarksdock.widget().reload_bookmarks()
 
         elif config == prefs.TEXTWIDTH:
-            # text width used for line wrapping
+            # Use the effective value for the same reason as tabwidth.
+            value = prefs.textwidth(self.context)
             self.commiteditor.set_textwidth(value)
 
         elif config == prefs.SHOW_PATH:
@@ -852,6 +884,7 @@ class MainView(standard.MainWindow):
         show_status_filter = self.statuswidget.filter_widget.isVisible()
         state['show_status_filter'] = show_status_filter
         state['toolbars'] = self.toolbar_state.export_state()
+        state['ref_sort'] = self.model.ref_sort
         self.diffviewer.export_state(state)
 
         return state
@@ -868,6 +901,9 @@ class MainView(standard.MainWindow):
         toolbars = state.get('toolbars', [])
         self.toolbar_state.apply_state(toolbars)
 
+        sort_key = state.get('ref_sort', 0)
+        self.model.set_ref_sort(sort_key)
+
         diff_ok = self.diffviewer.apply_state(state)
         return base_ok and diff_ok
 
@@ -885,7 +921,8 @@ class MainView(standard.MainWindow):
             (optkey + '+4', self.actionsdock),
             (optkey + '+5', self.bookmarksdock),
             (optkey + '+6', self.recentdock),
-            (optkey + '+7', self.branchdock)
+            (optkey + '+7', self.branchdock),
+            (optkey + '+8', self.submodulesdock)
         )
         for shortcut, dockwidget in dockwidgets:
             # Associate the action with the shortcut
