@@ -15,7 +15,6 @@ from .. import gitcmds
 from .. import icons
 from .. import qtutils
 from .. import utils
-from .standard import ProgressDialog
 from . import defs
 from . import standard
 
@@ -129,7 +128,7 @@ class RemoteActionDialog(standard.Dialog):
             self.setWindowModality(Qt.WindowModal)
 
         self.runtask = qtutils.RunTask(parent=self)
-        self.progress = ProgressDialog(title, N_('Updating'), self)
+        self.progress = standard.progress(title, N_('Updating'), self)
 
         self.local_label = QtWidgets.QLabel()
         self.local_label.setText(N_('Local Branch'))
@@ -264,7 +263,7 @@ class RemoteActionDialog(standard.Dialog):
         self.top_layout = qtutils.hbox(defs.no_margin, defs.spacing, *widgets)
 
         self.main_layout = qtutils.vbox(
-            defs.no_margin, defs.spacing,
+            defs.margin, defs.spacing,
             self.top_layout, self.options_layout)
         self.setLayout(self.main_layout)
 
@@ -337,7 +336,7 @@ class RemoteActionDialog(standard.Dialog):
         """Set sensible initial defaults"""
         # Default to "git fetch origin master"
         action = self.action
-        if action == FETCH or action == PULL:
+        if action in (FETCH, PULL):
             self.local_branch.setText('')
             self.remote_branch.setText('')
             return
@@ -392,6 +391,36 @@ class RemoteActionDialog(standard.Dialog):
             result = False
         return result
 
+    def select_remote_by_name(self, remote):
+        """Select a remote by name"""
+        remotes = self.model.remotes
+        if remote in remotes:
+            idx = remotes.index(remote)
+            result = self.select_remote(idx)
+        else:
+            result = False
+        return result
+
+    def set_selected_remotes(self, remotes):
+        """Set the list of selected remotes
+
+        Return True if all remotes were found and selected.
+
+        """
+        # Invalid remote names are ignored.
+        # This handles a remote going away between sessions.
+        # The selection is unchanged when none of the specified remotes exist.
+        found = False
+        for remote in remotes:
+            if remote in self.model.remotes:
+                found = True
+                break
+        if found:
+            # Only clear the selection if the specified remotes exist
+            self.remotes.clearSelection()
+            found = all([self.select_remote_by_name(x) for x in remotes])
+        return found
+
     def select_local_branch(self, idx):
         """Selects a local branch by index in the list view"""
         item = self.local_branches.item(idx)
@@ -639,19 +668,22 @@ class Push(RemoteActionDialog):
     def export_state(self):
         """Export persistent settings"""
         state = RemoteActionDialog.export_state(self)
-        state['tags'] = get(self.tags_checkbox)
         state['prompt'] = get(self.prompt_checkbox)
+        state['tags'] = get(self.tags_checkbox)
         return state
 
     def apply_state(self, state):
         """Apply persistent settings"""
         result = RemoteActionDialog.apply_state(self, state)
 
+        # Restore the "prompt on creation" checkbox
+        prompt = bool(state.get('prompt', True))
+        self.prompt_checkbox.setChecked(prompt)
+
+        # Restore the "tags" checkbox
         tags = bool(state.get('tags', False))
         self.tags_checkbox.setChecked(tags)
 
-        prompt = bool(state.get('prompt', True))
-        self.prompt_checkbox.setChecked(prompt)
         return result
 
 
